"""
Training strategies for deep learning
- author: Auorui (夏天是冰红茶)
- creation time:2023.9
- pyzjr.dlearn is a part of the model strategy used for deep learning
- WeChat: z15583909992
- Email: zjricetea@gmail.com
- Note: Currently still being updated, please refer to the latest version for any changes that may occur

语义分割训练模板
"""
from .activations import *
from .strategy import *
from .voc import *
from .callbacks import *
from .metrics import *
from .transforms import *
from .learnrate import *
from .tools import *


